@@ -46,9 +46,13 @@ typedef struct {
 
 typedef struct {
   IPAddr ip_addr;
-  IPAddr local_ip_addr;
   unsigned short port;
 } NTP_Remote_Address;
 
+typedef struct {
+  IPAddr ip_addr;
+  int sock_fd;
+} NTP_Local_Address;
+
 #endif /* GOT_ADDRESSING_H */
 
