@@ -3,6 +3,7 @@
 
  **********************************************************************
  * Copyright (C) Richard P. Curnow  1997-2002
+ * Copyright (C) Miroslav Lichvar  2013-2014
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -28,11 +29,35 @@
 #ifndef GOT_LOGGING_H
 #define GOT_LOGGING_H
 
+/* Flag indicating whether debug messages are logged */
+extern int log_debug_enabled;
+
+/* Line logging macros.  If the compiler is GNU C, we take advantage of
+   being able to get the function name also. */
+
+#ifdef __GNUC__
+#define FUNCTION_NAME __FUNCTION__
+#define FORMAT_ATTRIBUTE_PRINTF(str, first) __attribute__ ((format (printf, str, first)))
+#else
+#define FUNCTION_NAME ""
+#define FORMAT_ATTRIBUTE_PRINTF(str, first)
+#endif
+
+#define DEBUG_LOG(facility, ...) \
+  do { \
+    if (DEBUG && log_debug_enabled) \
+      LOG_Message(LOGS_DEBUG, facility, __LINE__, __FILE__, FUNCTION_NAME, __VA_ARGS__); \
+  } while (0)
+#define LOG(severity, facility, ...) LOG_Message(severity, facility, __LINE__, __FILE__, FUNCTION_NAME, __VA_ARGS__)
+#define LOG_FATAL(facility, ...) LOG_Message(LOGS_FATAL, facility, __LINE__, __FILE__, FUNCTION_NAME, __VA_ARGS__)
+
 /* Definition of severity */
 typedef enum {
   LOGS_INFO,
   LOGS_WARN,
-  LOGS_ERR
+  LOGS_ERR,
+  LOGS_FATAL,
+  LOGS_DEBUG
 } LOG_Severity;
 
 /* Definition of facility.  Each message is tagged with who generated
@@ -56,14 +81,17 @@ typedef enum {
   LOGF_Manual,
   LOGF_Keys,
   LOGF_Logging,
+  LOGF_Nameserv,
   LOGF_Rtc,
   LOGF_Regress,
   LOGF_Sys,
+  LOGF_SysGeneric,
   LOGF_SysLinux,
   LOGF_SysNetBSD,
   LOGF_SysSolaris,
   LOGF_SysSunOS,
   LOGF_SysWinnt,
+  LOGF_TempComp,
   LOGF_RtcLinux,
   LOGF_Refclock
 } LOG_Facility;
@@ -75,13 +103,17 @@ extern void LOG_Initialise(void);
 extern void LOG_Finalise(void);
 
 /* Line logging function */
-extern void LOG_Line_Function(LOG_Severity severity, LOG_Facility facility, const char *format, ...);
+FORMAT_ATTRIBUTE_PRINTF(6, 7)
+extern void LOG_Message(LOG_Severity severity, LOG_Facility facility,
+                        int line_number, const char *filename,
+                        const char *function_name, const char *format, ...);
 
-/* Logging function for fatal errors */
-extern void LOG_Fatal_Function(LOG_Facility facility, const char *format, ...);
-
-/* Position in code reporting function */
-extern void LOG_Position(const char *filename, int line_number, const char *function_name);
+/* Set debug level:
+   0, 1 - only non-debug messages are logged
+   2    - debug messages are logged too, all messages are prefixed with
+          filename, line, and function name
+   */
+extern void LOG_SetDebugLevel(int level);
 
 /* Log messages to syslog instead of stderr */
 extern void LOG_OpenSystemLog(void);
@@ -92,24 +124,13 @@ extern void LOG_SetParentFd(int fd);
 /* Close the pipe to the foreground process so it can exit */
 extern void LOG_CloseParentFd(void);
 
-/* Return zero once per 10 seconds */
-extern int LOG_RateLimited(void);
-
-/* Line logging macro.  If the compiler is GNU C, we take advantage of
-   being able to get the function name also. */
-#if defined(__GNUC__)
-#define LOG LOG_Position(__FILE__, __LINE__, __FUNCTION__); LOG_Line_Function
-#define LOG_FATAL LOG_Position(__FILE__, __LINE__, __FUNCTION__); LOG_Fatal_Function
-#else
-#define LOG LOG_Position(__FILE__, __LINE__, ""); LOG_Line_Function
-#define LOG_FATAL LOG_Position(__FILE__, __LINE__, ""); LOG_Fatal_Function
-#endif /* defined (__GNUC__) */
-
 /* File logging functions */
 
 typedef int LOG_FileID;
 
 extern LOG_FileID LOG_FileOpen(const char *name, const char *banner);
+
+FORMAT_ATTRIBUTE_PRINTF(2, 3)
 extern void LOG_FileWrite(LOG_FileID id, const char *format, ...);
 
 extern void LOG_CreateLogFileDir(void);
