@@ -4,7 +4,7 @@
  **********************************************************************
  * Copyright (C) Richard P. Curnow  1997-2003
  * Copyright (C) Timo Teras  2009
- * Copyright (C) Miroslav Lichvar  2009
+ * Copyright (C) Miroslav Lichvar  2009, 2013-2014
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -39,6 +39,8 @@
 #include "conf.h"
 #include "util.h"
 
+#define INVALID_SOCK_FD -1
+
 union sockaddr_in46 {
   struct sockaddr_in in4;
 #ifdef HAVE_IPV6
@@ -47,19 +49,24 @@ union sockaddr_in46 {
   struct sockaddr u;
 };
 
-/* The file descriptors for the IPv4 and IPv6 sockets */
-static int sock_fd4;
+/* The server/peer and client sockets for IPv4 and IPv6 */
+static int server_sock_fd4;
+static int client_sock_fd4;
 #ifdef HAVE_IPV6
-static int sock_fd6;
+static int server_sock_fd6;
+static int client_sock_fd6;
 #endif
 
+/* Flag indicating we create a new connected client socket for each
+   server instead of sharing client_sock_fd4 and client_sock_fd6 */
+static int separate_client_sockets;
+
 /* Flag indicating that we have been initialised */
 static int initialised=0;
 
 /* ================================================== */
 
 /* Forward prototypes */
-static int prepare_socket(int family);
 static void read_from_socket(void *anything);
 
 /* ================================================== */
@@ -92,17 +99,14 @@ do_size_checks(void)
 /* ================================================== */
 
 static int
-prepare_socket(int family)
+prepare_socket(int family, int port_number, int client_only)
 {
   union sockaddr_in46 my_addr;
   socklen_t my_addr_len;
   int sock_fd;
-  unsigned short port_number;
   IPAddr bind_address;
   int on_off = 1;
   
-  port_number = CNF_GetNTPPort();
-
   /* Open Internet domain UDP socket for NTP message transmissions */
 
 #if 0
@@ -114,20 +118,22 @@ prepare_socket(int family)
   if (sock_fd < 0) {
     LOG(LOGS_ERR, LOGF_NtpIO, "Could not open %s NTP socket : %s",
         family == AF_INET ? "IPv4" : "IPv6", strerror(errno));
-    return -1;
+    return INVALID_SOCK_FD;
   }
 
   /* Close on exec */
   UTI_FdSetCloexec(sock_fd);
 
-  /* Make the socket capable of re-using an old address */
-  if (setsockopt(sock_fd, SOL_SOCKET, SO_REUSEADDR, (char *)&on_off, sizeof(on_off)) < 0) {
+  /* Make the socket capable of re-using an old address if binding to a specific port */
+  if (port_number &&
+      setsockopt(sock_fd, SOL_SOCKET, SO_REUSEADDR, (char *)&on_off, sizeof(on_off)) < 0) {
     LOG(LOGS_ERR, LOGF_NtpIO, "Could not set reuseaddr socket options");
     /* Don't quit - we might survive anyway */
   }
   
   /* Make the socket capable of sending broadcast pkts - needed for NTP broadcast mode */
-  if (setsockopt(sock_fd, SOL_SOCKET, SO_BROADCAST, (char *)&on_off, sizeof(on_off)) < 0) {
+  if (!client_only &&
+      setsockopt(sock_fd, SOL_SOCKET, SO_BROADCAST, (char *)&on_off, sizeof(on_off)) < 0) {
     LOG(LOGS_ERR, LOGF_NtpIO, "Could not set broadcast socket options");
     /* Don't quit - we might survive anyway */
   }
@@ -142,8 +148,9 @@ prepare_socket(int family)
 
   if (family == AF_INET) {
 #ifdef IP_PKTINFO
-    /* We want the local IP info too */
-    if (setsockopt(sock_fd, IPPROTO_IP, IP_PKTINFO, (char *)&on_off, sizeof(on_off)) < 0) {
+    /* We want the local IP info on server sockets */
+    if (!client_only &&
+        setsockopt(sock_fd, IPPROTO_IP, IP_PKTINFO, (char *)&on_off, sizeof(on_off)) < 0) {
       LOG(LOGS_ERR, LOGF_NtpIO, "Could not request packet info using socket option");
       /* Don't quit - we might survive anyway */
     }
@@ -158,15 +165,17 @@ prepare_socket(int family)
     }
 #endif
 
+    if (!client_only) {
 #ifdef IPV6_RECVPKTINFO
-    if (setsockopt(sock_fd, IPPROTO_IPV6, IPV6_RECVPKTINFO, (char *)&on_off, sizeof(on_off)) < 0) {
-      LOG(LOGS_ERR, LOGF_NtpIO, "Could not request IPv6 packet info socket option");
-    }
+      if (setsockopt(sock_fd, IPPROTO_IPV6, IPV6_RECVPKTINFO, (char *)&on_off, sizeof(on_off)) < 0) {
+        LOG(LOGS_ERR, LOGF_NtpIO, "Could not request IPv6 packet info socket option");
+      }
 #elif defined(IPV6_PKTINFO)
-    if (setsockopt(sock_fd, IPPROTO_IPV6, IPV6_PKTINFO, (char *)&on_off, sizeof(on_off)) < 0) {
-      LOG(LOGS_ERR, LOGF_NtpIO, "Could not request IPv6 packet info socket option");
-    }
+      if (setsockopt(sock_fd, IPPROTO_IPV6, IPV6_PKTINFO, (char *)&on_off, sizeof(on_off)) < 0) {
+        LOG(LOGS_ERR, LOGF_NtpIO, "Could not request IPv6 packet info socket option");
+      }
 #endif
+    }
   }
 #endif
 
@@ -179,7 +188,10 @@ prepare_socket(int family)
       my_addr.in4.sin_family = family;
       my_addr.in4.sin_port = htons(port_number);
 
-      CNF_GetBindAddress(IPADDR_INET4, &bind_address);
+      if (!client_only)
+        CNF_GetBindAddress(IPADDR_INET4, &bind_address);
+      else
+        CNF_GetBindAcquisitionAddress(IPADDR_INET4, &bind_address);
 
       if (bind_address.family == IPADDR_INET4)
         my_addr.in4.sin_addr.s_addr = htonl(bind_address.addr.in4);
@@ -192,7 +204,10 @@ prepare_socket(int family)
       my_addr.in6.sin6_family = family;
       my_addr.in6.sin6_port = htons(port_number);
 
-      CNF_GetBindAddress(IPADDR_INET6, &bind_address);
+      if (!client_only)
+        CNF_GetBindAddress(IPADDR_INET6, &bind_address);
+      else
+        CNF_GetBindAcquisitionAddress(IPADDR_INET6, &bind_address);
 
       if (bind_address.family == IPADDR_INET6)
         memcpy(my_addr.in6.sin6_addr.s6_addr, bind_address.addr.in6,
@@ -205,15 +220,11 @@ prepare_socket(int family)
       assert(0);
   }
 
-#if 0
-  LOG(LOGS_INFO, LOGF_NtpIO, "Initialising, socket fd=%d", sock_fd);
-#endif
-
   if (bind(sock_fd, &my_addr.u, my_addr_len) < 0) {
     LOG(LOGS_ERR, LOGF_NtpIO, "Could not bind %s NTP socket : %s",
         family == AF_INET ? "IPv4" : "IPv6", strerror(errno));
     close(sock_fd);
-    return -1;
+    return INVALID_SOCK_FD;
   }
 
   /* Register handler for read events on the socket */
@@ -233,31 +244,114 @@ prepare_socket(int family)
 
 /* ================================================== */
 
+static int
+connect_socket(int sock_fd, NTP_Remote_Address *remote_addr)
+{
+  union sockaddr_in46 addr;
+  socklen_t addr_len;
+
+  memset(&addr, 0, sizeof (addr));
+
+  switch (remote_addr->ip_addr.family) {
+    case IPADDR_INET4:
+      addr_len = sizeof (addr.in4);
+      addr.in4.sin_family = AF_INET;
+      addr.in4.sin_addr.s_addr = htonl(remote_addr->ip_addr.addr.in4);
+      addr.in4.sin_port = htons(remote_addr->port);
+      break;
+#ifdef HAVE_IPV6
+    case IPADDR_INET6:
+      addr_len = sizeof (addr.in6);
+      addr.in6.sin6_family = AF_INET6;
+      memcpy(addr.in6.sin6_addr.s6_addr, remote_addr->ip_addr.addr.in6,
+             sizeof (addr.in6.sin6_addr.s6_addr));
+      addr.in6.sin6_port = htons(remote_addr->port);
+      break;
+#endif
+    default:
+      assert(0);
+  }
+
+  if (connect(sock_fd, &addr.u, addr_len) < 0) {
+    LOG(LOGS_ERR, LOGF_NtpIO, "Could not connect NTP socket to %s:%d : %s",
+        UTI_IPToString(&remote_addr->ip_addr), remote_addr->port,
+        strerror(errno));
+    return 0;
+  }
+
+  return 1;
+}
+
+/* ================================================== */
+
+static void
+close_socket(int sock_fd)
+{
+  if (sock_fd == INVALID_SOCK_FD)
+    return;
+
+  SCH_RemoveInputFileHandler(sock_fd);
+  close(sock_fd);
+}
+
+/* ================================================== */
+
 void
 NIO_Initialise(int family)
 {
+  int server_port, client_port;
+
   assert(!initialised);
   initialised = 1;
 
   do_size_checks();
 
-  if (family == IPADDR_UNSPEC || family == IPADDR_INET4)
-    sock_fd4 = prepare_socket(AF_INET);
-  else
-    sock_fd4 = -1;
+  server_port = CNF_GetNTPPort();
+  client_port = CNF_GetAcquisitionPort();
+
+  /* Use separate connected sockets if client port is not set */
+  separate_client_sockets = client_port == 0;
+
+  server_sock_fd4 = INVALID_SOCK_FD;
+  client_sock_fd4 = INVALID_SOCK_FD;
 #ifdef HAVE_IPV6
-  if (family == IPADDR_UNSPEC || family == IPADDR_INET6)
-    sock_fd6 = prepare_socket(AF_INET6);
-  else
-    sock_fd6 = -1;
+  server_sock_fd6 = INVALID_SOCK_FD;
+  client_sock_fd6 = INVALID_SOCK_FD;
 #endif
 
-  if (sock_fd4 < 0
+  if (family == IPADDR_UNSPEC || family == IPADDR_INET4) {
+    if (server_port)
+      server_sock_fd4 = prepare_socket(AF_INET, server_port, 0);
+    if (!separate_client_sockets) {
+      if (client_port != server_port || !server_port)
+        client_sock_fd4 = prepare_socket(AF_INET, client_port, 1);
+      else
+        client_sock_fd4 = server_sock_fd4;
+    }
+  }
 #ifdef HAVE_IPV6
-      && sock_fd6 < 0
+  if (family == IPADDR_UNSPEC || family == IPADDR_INET6) {
+    if (server_port)
+      server_sock_fd6 = prepare_socket(AF_INET6, server_port, 0);
+    if (!separate_client_sockets) {
+      if (client_port != server_port || !server_port)
+        client_sock_fd6 = prepare_socket(AF_INET6, client_port, 1);
+      else
+        client_sock_fd6 = server_sock_fd6;
+    }
+  }
 #endif
-      ) {
-    LOG_FATAL(LOGF_NtpIO, "Could not open any NTP socket");
+
+  if ((server_port && server_sock_fd4 == INVALID_SOCK_FD
+#ifdef HAVE_IPV6
+       && server_sock_fd6 == INVALID_SOCK_FD
+#endif
+      ) || (!separate_client_sockets && client_sock_fd4 == INVALID_SOCK_FD
+#ifdef HAVE_IPV6
+       && client_sock_fd6 == INVALID_SOCK_FD
+#endif
+      )) {
+    LOG_FATAL(LOGF_NtpIO, "Could not open NTP sockets");
   }
 }
 
@@ -266,23 +360,101 @@ NIO_Initialise(int family)
 void
 NIO_Finalise(void)
 {
-  if (sock_fd4 >= 0) {
-    SCH_RemoveInputFileHandler(sock_fd4);
-    close(sock_fd4);
-  }
-  sock_fd4 = -1;
+  if (server_sock_fd4 != client_sock_fd4)
+    close_socket(client_sock_fd4);
+  close_socket(server_sock_fd4);
+  server_sock_fd4 = client_sock_fd4 = INVALID_SOCK_FD;
 #ifdef HAVE_IPV6
-  if (sock_fd6 >= 0) {
-    SCH_RemoveInputFileHandler(sock_fd6);
-    close(sock_fd6);
-  }
-  sock_fd6 = -1;
+  if (server_sock_fd6 != client_sock_fd6)
+    close_socket(client_sock_fd6);
+  close_socket(server_sock_fd6);
+  server_sock_fd6 = client_sock_fd6 = INVALID_SOCK_FD;
 #endif
   initialised = 0;
 }
 
 /* ================================================== */
 
+int
+NIO_GetClientSocket(NTP_Remote_Address *remote_addr)
+{
+  if (separate_client_sockets) {
+    int sock_fd;
+
+    switch (remote_addr->ip_addr.family) {
+      case IPADDR_INET4:
+        sock_fd = prepare_socket(AF_INET, 0, 1);
+        break;
+#ifdef HAVE_IPV6
+      case IPADDR_INET6:
+        sock_fd = prepare_socket(AF_INET6, 0, 1);
+        break;
+#endif
+      default:
+        sock_fd = INVALID_SOCK_FD;
+    }
+
+    if (sock_fd == INVALID_SOCK_FD)
+      return INVALID_SOCK_FD;
+
+    if (!connect_socket(sock_fd, remote_addr)) {
+      close_socket(sock_fd);
+      return INVALID_SOCK_FD;
+    }
+
+    return sock_fd;
+  } else {
+    switch (remote_addr->ip_addr.family) {
+      case IPADDR_INET4:
+        return client_sock_fd4;
+#ifdef HAVE_IPV6
+      case IPADDR_INET6:
+        return client_sock_fd6;
+#endif
+      default:
+        return INVALID_SOCK_FD;
+    }
+  }
+}
+
+/* ================================================== */
+
+int
+NIO_GetServerSocket(NTP_Remote_Address *remote_addr)
+{
+  switch (remote_addr->ip_addr.family) {
+    case IPADDR_INET4:
+      return server_sock_fd4;
+#ifdef HAVE_IPV6
+    case IPADDR_INET6:
+      return server_sock_fd6;
+#endif
+    default:
+      return INVALID_SOCK_FD;
+  }
+}
+
+/* ================================================== */
+
+void
+NIO_CloseClientSocket(int sock_fd)
+{
+  if (separate_client_sockets)
+    close_socket(sock_fd);
+}
+
+/* ================================================== */
+
+int
+NIO_IsServerSocket(int sock_fd)
+{
+  return sock_fd != INVALID_SOCK_FD &&
+    (sock_fd == server_sock_fd4
+#ifdef HAVE_IPV6
+     || sock_fd == server_sock_fd6
+#endif
+    );
+}
 
 /* ================================================== */
 
@@ -296,9 +468,10 @@ read_from_socket(void *anything)
   ReceiveBuffer message;
   union sockaddr_in46 where_from;
   unsigned int flags = 0;
-  struct timeval now, now_raw;
+  struct timeval now;
   double now_err;
   NTP_Remote_Address remote_addr;
+  NTP_Local_Address local_addr;
   char cmsgbuf[256];
   struct msghdr msg;
   struct iovec iov;
@@ -306,7 +479,7 @@ read_from_socket(void *anything)
 
   assert(initialised);
 
-  SCH_GetLastEventTime(&now, &now_err, &now_raw);
+  SCH_GetLastEventTime(&now, &now_err, NULL);
 
   iov.iov_base = message.arbitrary;
   iov.iov_len = sizeof(message);
@@ -329,8 +502,6 @@ read_from_socket(void *anything)
      reponse on a subsequent recvfrom). */
 
   if (status > 0) {
-    memset(&remote_addr, 0, sizeof (remote_addr));
-
     switch (where_from.u.sa_family) {
       case AF_INET:
         remote_addr.ip_addr.family = IPADDR_INET4;
@@ -349,14 +520,17 @@ read_from_socket(void *anything)
         assert(0);
     }
 
+    local_addr.ip_addr.family = IPADDR_UNSPEC;
+    local_addr.sock_fd = sock_fd;
+
     for (cmsg = CMSG_FIRSTHDR(&msg); cmsg; cmsg = CMSG_NXTHDR(&msg, cmsg)) {
 #ifdef IP_PKTINFO
       if (cmsg->cmsg_level == IPPROTO_IP && cmsg->cmsg_type == IP_PKTINFO) {
         struct in_pktinfo ipi;
 
         memcpy(&ipi, CMSG_DATA(cmsg), sizeof(ipi));
-        remote_addr.local_ip_addr.addr.in4 = ntohl(ipi.ipi_spec_dst.s_addr);
-        remote_addr.local_ip_addr.family = IPADDR_INET4;
+        local_addr.ip_addr.addr.in4 = ntohl(ipi.ipi_spec_dst.s_addr);
+        local_addr.ip_addr.family = IPADDR_INET4;
       }
 #endif
 
@@ -365,9 +539,9 @@ read_from_socket(void *anything)
         struct in6_pktinfo ipi;
 
         memcpy(&ipi, CMSG_DATA(cmsg), sizeof(ipi));
-        memcpy(&remote_addr.local_ip_addr.addr.in6, &ipi.ipi6_addr.s6_addr,
-            sizeof (remote_addr.local_ip_addr.addr.in6));
-        remote_addr.local_ip_addr.family = IPADDR_INET6;
+        memcpy(&local_addr.ip_addr.addr.in6, &ipi.ipi6_addr.s6_addr,
+            sizeof (local_addr.ip_addr.addr.in6));
+        local_addr.ip_addr.family = IPADDR_INET6;
       }
 #endif
 
@@ -376,16 +550,15 @@ read_from_socket(void *anything)
         struct timeval tv;
 
         memcpy(&tv, CMSG_DATA(cmsg), sizeof(tv));
-
-        /* This should be more accurate than LCL_CookTime(&now_raw,...) */
-        UTI_AddDiffToTimeval(&now, &now_raw, &tv, &now);
+        LCL_CookTime(&tv, &now, &now_err);
       }
 #endif
     }
 
     if (status >= NTP_NORMAL_PACKET_SIZE && status <= sizeof(NTP_Packet)) {
 
-      NSR_ProcessReceive((NTP_Packet *) &message.ntp_pkt, &now, now_err, &remote_addr, status);
+      NSR_ProcessReceive((NTP_Packet *) &message.ntp_pkt, &now, now_err,
+                         &remote_addr, &local_addr, status);
 
     } else {
 
@@ -399,49 +572,61 @@ read_from_socket(void *anything)
 /* Send a packet to given address */
 
 static void
-send_packet(void *packet, int packetlen, NTP_Remote_Address *remote_addr)
+send_packet(void *packet, int packetlen, NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr)
 {
   union sockaddr_in46 remote;
   struct msghdr msg;
   struct iovec iov;
   char cmsgbuf[256];
   int cmsglen;
-  int sock_fd;
-  socklen_t addrlen;
+  socklen_t addrlen = 0;
 
   assert(initialised);
 
+  if (local_addr->sock_fd == INVALID_SOCK_FD) {
+    DEBUG_LOG(LOGF_NtpIO, "No socket to send to %s:%d",
+              UTI_IPToString(&remote_addr->ip_addr), remote_addr->port);
+    return;
+  }
+
   switch (remote_addr->ip_addr.family) {
     case IPADDR_INET4:
+      /* Don't set address with connected socket */
+      if (local_addr->sock_fd != server_sock_fd4 && separate_client_sockets)
+        break;
       memset(&remote.in4, 0, sizeof (remote.in4));
       addrlen = sizeof (remote.in4);
       remote.in4.sin_family = AF_INET;
       remote.in4.sin_port = htons(remote_addr->port);
       remote.in4.sin_addr.s_addr = htonl(remote_addr->ip_addr.addr.in4);
-      sock_fd = sock_fd4;
       break;
 #ifdef HAVE_IPV6
     case IPADDR_INET6:
+      /* Don't set address with connected socket */
+      if (local_addr->sock_fd != server_sock_fd6 && separate_client_sockets)
+        break;
       memset(&remote.in6, 0, sizeof (remote.in6));
       addrlen = sizeof (remote.in6);
       remote.in6.sin6_family = AF_INET6;
       remote.in6.sin6_port = htons(remote_addr->port);
       memcpy(&remote.in6.sin6_addr.s6_addr, &remote_addr->ip_addr.addr.in6,
           sizeof (remote.in6.sin6_addr.s6_addr));
-      sock_fd = sock_fd6;
       break;
 #endif
     default:
       return;
   }
 
-  if (sock_fd < 0)
-    return;
+  if (addrlen) {
+    msg.msg_name = &remote.u;
+    msg.msg_namelen = addrlen;
+  } else {
+    msg.msg_name = NULL;
+    msg.msg_namelen = 0;
+  }
 
   iov.iov_base = packet;
   iov.iov_len = packetlen;
-  msg.msg_name = &remote.u;
-  msg.msg_namelen = addrlen;
   msg.msg_iov = &iov;
   msg.msg_iovlen = 1;
   msg.msg_control = cmsgbuf;
@@ -450,7 +635,7 @@ send_packet(void *packet, int packetlen, NTP_Remote_Address *remote_addr)
   cmsglen = 0;
 
 #ifdef IP_PKTINFO
-  if (remote_addr->local_ip_addr.family == IPADDR_INET4) {
+  if (local_addr->ip_addr.family == IPADDR_INET4) {
     struct cmsghdr *cmsg;
     struct in_pktinfo *ipi;
 
@@ -463,12 +648,12 @@ send_packet(void *packet, int packetlen, NTP_Remote_Address *remote_addr)
     cmsg->cmsg_len = CMSG_LEN(sizeof(struct in_pktinfo));
 
     ipi = (struct in_pktinfo *) CMSG_DATA(cmsg);
-    ipi->ipi_spec_dst.s_addr = htonl(remote_addr->local_ip_addr.addr.in4);
+    ipi->ipi_spec_dst.s_addr = htonl(local_addr->ip_addr.addr.in4);
   }
 #endif
 
 #if defined(IPV6_PKTINFO) && defined(HAVE_IN6_PKTINFO)
-  if (remote_addr->local_ip_addr.family == IPADDR_INET6) {
+  if (local_addr->ip_addr.family == IPADDR_INET6) {
     struct cmsghdr *cmsg;
     struct in6_pktinfo *ipi;
 
@@ -481,30 +666,22 @@ send_packet(void *packet, int packetlen, NTP_Remote_Address *remote_addr)
     cmsg->cmsg_len = CMSG_LEN(sizeof(struct in6_pktinfo));
 
     ipi = (struct in6_pktinfo *) CMSG_DATA(cmsg);
-    memcpy(&ipi->ipi6_addr.s6_addr, &remote_addr->local_ip_addr.addr.in6,
+    memcpy(&ipi->ipi6_addr.s6_addr, &local_addr->ip_addr.addr.in6,
         sizeof(ipi->ipi6_addr.s6_addr));
   }
 #endif
 
-#if 0
-    LOG(LOGS_INFO, LOGF_NtpIO, "sending to %s:%d from %s",
-        UTI_IPToString(&remote_addr->ip_addr), remote_addr->port, UTI_IPToString(&remote_addr->local_ip_addr));
-#endif
+  DEBUG_LOG(LOGF_NtpIO, "Sending to %s:%d from %s fd %d",
+      UTI_IPToString(&remote_addr->ip_addr), remote_addr->port,
+      UTI_IPToString(&local_addr->ip_addr), local_addr->sock_fd);
 
   msg.msg_controllen = cmsglen;
   /* This is apparently required on some systems */
   if (!cmsglen)
     msg.msg_control = NULL;
 
-  if (sendmsg(sock_fd, &msg, 0) < 0 &&
-#ifdef ENETUNREACH
-      errno != ENETUNREACH &&
-#endif
-#ifdef ENETDOWN
-      errno != ENETDOWN &&
-#endif
-      !LOG_RateLimited()) {
-    LOG(LOGS_WARN, LOGF_NtpIO, "Could not send to %s:%d : %s",
+  if (sendmsg(local_addr->sock_fd, &msg, 0) < 0) {
+    DEBUG_LOG(LOGF_NtpIO, "Could not send to %s:%d : %s",
         UTI_IPToString(&remote_addr->ip_addr), remote_addr->port, strerror(errno));
   }
 }
@@ -513,18 +690,18 @@ send_packet(void *packet, int packetlen, NTP_Remote_Address *remote_addr)
 /* Send an unauthenticated packet to a given address */
 
 void
-NIO_SendNormalPacket(NTP_Packet *packet, NTP_Remote_Address *remote_addr)
+NIO_SendNormalPacket(NTP_Packet *packet, NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr)
 {
-  send_packet((void *) packet, NTP_NORMAL_PACKET_SIZE, remote_addr);
+  send_packet((void *) packet, NTP_NORMAL_PACKET_SIZE, remote_addr, local_addr);
 }
 
 /* ================================================== */
 /* Send an authenticated packet to a given address */
 
 void
-NIO_SendAuthenticatedPacket(NTP_Packet *packet, NTP_Remote_Address *remote_addr, int auth_len)
+NIO_SendAuthenticatedPacket(NTP_Packet *packet, NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr, int auth_len)
 {
-  send_packet((void *) packet, NTP_NORMAL_PACKET_SIZE + auth_len, remote_addr);
+  send_packet((void *) packet, NTP_NORMAL_PACKET_SIZE + auth_len, remote_addr, local_addr);
 }
 
 /* ================================================== */
@@ -533,7 +710,7 @@ NIO_SendAuthenticatedPacket(NTP_Packet *packet, NTP_Remote_Address *remote_addr,
 #define ECHO_PORT 7
 
 void
-NIO_SendEcho(NTP_Remote_Address *remote_addr)
+NIO_SendEcho(NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr)
 {
   unsigned long magic_message = 0xbe7ab1e7UL;
   NTP_Remote_Address addr;
@@ -541,5 +718,5 @@ NIO_SendEcho(NTP_Remote_Address *remote_addr)
   addr = *remote_addr;
   addr.port = ECHO_PORT;
 
-  send_packet((void *) &magic_message, sizeof(unsigned long), &addr);
+  send_packet((void *) &magic_message, sizeof(unsigned long), &addr, local_addr);
 }
