@@ -3,7 +3,7 @@
 
  **********************************************************************
  * Copyright (C) Richard P. Curnow  1997-2003
- * Copyright (C) Miroslav Lichvar  2011
+ * Copyright (C) Miroslav Lichvar  2011, 2013-2014
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -123,7 +123,7 @@ handle_slew(struct timeval *raw,
             struct timeval *cooked,
             double dfreq,
             double doffset,
-            int is_step_change,
+            LCL_ChangeType change_type,
             void *anything);
 
 /* ================================================== */
@@ -503,14 +503,14 @@ handle_slew(struct timeval *raw,
             struct timeval *cooked,
             double dfreq,
             double doffset,
-            int is_step_change,
+            LCL_ChangeType change_type,
             void *anything)
 {
   TimerQueueEntry *ptr;
   double delta;
   int i;
 
-  if (is_step_change) {
+  if (change_type != LCL_ChangeAdjust) {
     /* If a step change occurs, just shift all raw time stamps by the offset */
     
     for (ptr = timer_queue.next; ptr != &timer_queue; ptr = ptr->next) {
@@ -529,30 +529,33 @@ handle_slew(struct timeval *raw,
 
 /* ================================================== */
 
-/* Try to handle unexpected backward time jump */
+#define JUMP_DETECT_THRESHOLD 10
 
-static void
-recover_backjump(struct timeval *raw, struct timeval *cooked, int timeout)
+static int
+check_current_time(struct timeval *raw, int timeout)
 {
-      double diff, err;
+  double diff;
 
-      UTI_DiffTimevalsToDouble(&diff, &last_select_ts_raw, raw);
+  if (last_select_ts_raw.tv_sec > raw->tv_sec + JUMP_DETECT_THRESHOLD) {
+    LOG(LOGS_WARN, LOGF_Scheduler, "Backward time jump detected!");
+  } else if (n_timer_queue_entries > 0 &&
+      timer_queue.next->tv.tv_sec + JUMP_DETECT_THRESHOLD < raw->tv_sec) {
+    LOG(LOGS_WARN, LOGF_Scheduler, "Forward time jump detected!");
+  } else {
+    return 1;
+  }
 
-      if (n_timer_queue_entries > 0) {
-        UTI_DiffTimevalsToDouble(&err, &(timer_queue.next->tv), &last_select_ts_raw);
-      } else {
-        err = 0.0;
-      }
+  if (timeout) {
+    assert(n_timer_queue_entries > 0);
+    UTI_DiffTimevalsToDouble(&diff, &timer_queue.next->tv, raw);
+  } else {
+    UTI_DiffTimevalsToDouble(&diff, &last_select_ts_raw, raw);
+  }
 
-      diff += err;
+  /* Cooked time may no longer be valid after dispatching the handlers */
+  LCL_NotifyExternalTimeStep(raw, raw, diff, fabs(diff));
 
-      if (timeout) {
-        err = 1.0;
-      }
-
-      LOG(LOGS_WARN, LOGF_Scheduler, "Backward time jump detected! (correction %.1f +- %.1f seconds)", diff, err);
-
-      LCL_NotifyExternalTimeStep(raw, cooked, diff, err);
+  return 0;
 }
 
 /* ================================================== */
@@ -569,13 +572,13 @@ SCH_MainLoop(void)
   assert(initialised);
 
   while (!need_to_exit) {
-
-    /* Copy current set of read file descriptors */
-    memcpy((void *) &rd, (void *) &read_fds, sizeof(fd_set));
-    
     /* Dispatch timeouts and fill now with current raw time */
     dispatch_timeouts(&now);
     
+    /* The timeout handlers may request quit */
+    if (need_to_exit)
+      break;
+
     /* Check whether there is a timeout and set it up */
     if (n_timer_queue_entries > 0) {
 
@@ -589,7 +592,12 @@ SCH_MainLoop(void)
 
     /* if there are no file descriptors being waited on and no
        timeout set, this is clearly ridiculous, so stop the run */
-    assert(ptv || n_read_fds);
+    if (!ptv && !n_read_fds) {
+      LOG_FATAL(LOGF_Scheduler, "Nothing to do");
+    }
+
+    /* Copy current set of read file descriptors */
+    memcpy((void *) &rd, (void *) &read_fds, sizeof(fd_set));
 
     status = select(one_highest_fd, &rd, NULL, NULL, ptv);
     errsv = errno;
@@ -597,9 +605,10 @@ SCH_MainLoop(void)
     LCL_ReadRawTime(&now);
     LCL_CookTime(&now, &cooked, &err);
 
-    /* Check if time didn't jump backwards */
-    if (last_select_ts_raw.tv_sec > now.tv_sec + 1) {
-      recover_backjump(&now, &cooked, status == 0);
+    /* Check if the time didn't jump unexpectedly */
+    if (!check_current_time(&now, status == 0)) {
+      /* Cook the time again after handling the step */
+      LCL_CookTime(&now, &cooked, &err);
     }
 
     last_select_ts_raw = now;
